define((()=>(()=>{"use strict";var e={d:(t,r)=>{for(var o in r)e.o(r,o)&&!e.o(t,o)&&Object.defineProperty(t,o,{enumerable:!0,get:r[o]})},o:(e,t)=>Object.prototype.hasOwnProperty.call(e,t),r:e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},p:""},t={};function r(){}return e.r(t),e.d(t,{load_ipython_extension:()=>r}),e.p=document.querySelector("body").getAttribute("data-base-url")+"nbextensions/cad-viewer-widget",window.require&&window.require.config({map:{"*":{"cad-viewer-widget":"nbextensions/cad-viewer-widget/index"}}}),t})()));